/*
 * (./) MachineMotsMouvants.java 0.1 05/11/03
 * (by) cousot stéphane @www.ubaa.net
 * (cc) some right reserved
 *
 * ALL FILES IN THIS PROJECT ARE RELEASED UNDER A CREATIVE COMMONS LICENSE BY_SA
 * http://creativecommons.org/licenses/by-sa/2.0/
 *
 *
 * "Machine des Mots Mouvants"(MMM) is developed for the artwork "Revenir d'Absence"
 * by Giannarelli Thierry, "L'Imparfait" company, La Colle de Nouvé, Cabasse,
 * France.
 *
 * To run properly, this project require two Java extensions: The Java Media 
 * Framework (JMF) available at http://java.sun.com/products/java-media/jmf/ and
 * The GNU Java Getopt available at http://www.urbanophile.com/arenn/hacking/ .
 *
 * usage : `java -Djava.library.path=./lib -jar MMM.jar [-p|-e] files...'
 * Use the `p' option to run the application in play mode or the `e' option for 
 * the editor mode. To launch directly one (or more in editor mode) project pass
 * the file path as arguments.
 */


import java.util.Vector;
import java.io.File;
import java.io.IOException;
import java.util.jar.*;

import gnu.getopt.Getopt;

import mmm.event.WindowManager;
import mmm.ui.Editor;
import mmm.ui.Player;


/* Parse the arguments and set the application 
 * in the play or editor mode
 */
public class MMM {
	
	
	// to be set with the relative Manifest attributes
	private static String version	= ""; 
	private static String title		= "";
	
	// create the application window manager
	public static WindowManager wm	= new WindowManager();
	
	
	/* delete folder path, non-existent file, non-file project
	 * and convert on the fly each valid path into a File object
	 * @arg  Vector, a list of file path
	 * @type Vector
	 */
	private static Vector checkFiles( Vector files ) {
		for(int i=(files.size()-1); i>=0; i--) {
			File f = new File( (String)files.get(i) );
			try {
				// remove folder and non-existent file
				if ( !f.exists() || f.isDirectory() ) {
					files.remove(i);
					System.out.println("invalid file path: " + 
									   f.getCanonicalFile());
				}
				// convert String > File
				else files.set(i, f);
			}
			catch(IOException e) {;}
		}
		return files;
	}
	
	/* get current Jar Manifest content
	 */
	private static void getJarAttributes() {
		try {
			JarFile jar		= new JarFile( "MMM.jar" );
			Manifest man	= jar.getManifest();
			Attributes attr	= man.getMainAttributes();
			
			version = attr.getValue( Attributes.Name.SPECIFICATION_VERSION );
			title	= attr.getValue( Attributes.Name.SPECIFICATION_TITLE );
		}
		catch( IOException e) { System.out.println( e.getMessage() ); }
	}
	
	/* output application informations
	 */
	private static void info() {
		System.out.println("\n// MMM v" + version + "\n// " + title + "\n" +
						   "// source code available at " +
						   "http://palimpseste.free.fr/project/revenir_absence/"+
						   "\n// (cc) some right reserved\n");
	}
	
	/* usage
	 */
	private static void usage() {
		
		String s =	"usage: " +
					"java -Djava.library.path=./lib -jar MMM.jar " +
					"[-p|-e] files...\n" +
					"\t-p\tplay mode, at least one file is required, " +
					"pass it as an argument\n" +
					"\t-e\teditor mode\n" + 
					"\t-v\tprint version\n" +
					"\t-h\tprint help\n";

		System.out.println(s);
		System.exit(0);
	}
	
	/* main method
	 * parse arguments and start this application
     * @param argument, a list of the project's files
	 *					and the application options
	 */
	public static void main( String[] args ) {
		
		int c, i;
		String arg;
		Getopt g	 = new Getopt("Machine des Mots Mouvants", args, "e::p:vh");
		Vector files = new Vector();
		String mode  = "editor";
		
		// retrieve manifest attributes
		getJarAttributes();
		
		// print usage
		if ( args.length==0 ) usage();
		
		// parse options
		while ( (c=g.getopt()) != -1 ) {
			switch(c) {
				
				case 'e':
					mode = "editor";
					arg  = g.getOptarg();
					if (arg!=null) files.add(arg);
					break;
					
				case 'p':
					mode = "player";
					arg  = g.getOptarg();
					if (arg!=null) files.add(arg);
					break;
				
				case 'v':
					System.out.println( "Machine des Mots Mouvants (MMM) "+
										"version : " + version );
					System.exit(0);
					break;
					
				case 'h':
					info();
				case '?':
					usage();
					break;
			}
		}
		
		// retrieve non-option arguments
		for(i=g.getOptind(); i<args.length; i++) files.add(args[i]);
		
		// start application
		info();
		
		// clean and convert paths
		files = checkFiles(files);
		
		// player mode
		if ( mode.equals("player") && files.size()!=0 )
			new Player( files.toArray() );
		
		// editor mode
		if ( mode.equals("editor") ) {
			if (files.size()==0) new Editor( wm );
			else for(i=0; i<files.size(); i++)
				new Editor( wm, (File)files.get(i) );
		}
	}
}


