/*
 * (./) Project.java 05/11/03 by cs for "MMM" project
 * (cc) some right reserved
 */

package mmm;

import java.awt.Color;
import java.io.File;
import java.util.Vector;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Date;

import java.net.*;
import java.io.*;

import mmm.io.XML;
import mmm.ui.Toolbar;

/* The base class for all projects,
 */
public class Project {
	
	// define the default project color
	public final static Color DEFAULT_TRACK_COLOR	= new Color( 198, 202, 196 );
	public final static Color COLOR_GREEN			= new Color( 0, 153, 0 );
	public final static Color COLOR_YELLOW			= new Color( 205, 203, 0 );
	public final static Color COLOR_PURPLE			= new Color( 159, 0, 179 );
	public final static Color COLOR_BLUE			= new Color( 153, 153, 255 );
	
	// fields
	public final static String DEFAULT_PROJECT_NAME	= "Untitled Project";
	public final static String DEFAULT_TRACK_NAME	= "output";
	public final static double DEFAULT_TRACK_TIME	= (7*60);
	public final static long TIMER_PERIOD			= 1000l;
	
	// project (file) properties
	private File file		= null;
	private File video		= null;
	private String name		= DEFAULT_PROJECT_NAME;
	
	// project duration
	private double duration = DEFAULT_TRACK_TIME;
	private double chrono	= 0;
	
	// the project's tracks list
	private Vector tracks	= new Vector();
	
	// the runnable timer
	private Timer timer		= null;
	
	// output message to the PCI status
	private static boolean toPCI = false;
	
	// librar
	private static boolean isLIB = false;
	
	// <--TMP--> UDP require variables
	static DatagramSocket socket	= null;
	static final int PORT			= 7000;  

	////////////////// JNI REQUIRED ////////////
	/*
	* load the required library, this library communicate directly with the PCI
	* bus to send informations to the vibrators.This application can run without
	* this library but can't output any messages.
	*/
    static {
		try {
			
			// launch LabVIEW application (windows)
			if ( System.getProperty("os.name").indexOf("Windows")!=-1 ) {
				try {
					System.out.print( "open LabVIEW Runtime ... " );
					Runtime.getRuntime().exec("cmd /K lv.exe");
					System.out.println( "ok" );
				}
				catch( Exception e ) { System.out.println( e.getMessage() ); }
			}
			
			// create UDP socket and reset all outputs
			try {
				System.out.print( "open UDP socket ... " );
				socket = new DatagramSocket( PORT+100 );
				System.out.println( "ok" );
				reset();
			}
			catch( SocketException e ) { System.out.println( e.getMessage() ); }
			
			// load Native library
			System.out.print( "load native library ... " );
			System.loadLibrary( "NI_PCI-6503" );
			System.out.println( "ok" );
			isLIB = true;
			//toPCI = isPCICard();
		}
		catch( UnsatisfiedLinkError e ) {
			
			String s = "failed!\n\n*** WARNING ***\n\""+
			System.mapLibraryName("NI_PCI-6503")+"\" library could not be "+
			"found.\nVerify that this library is present in the \"lib\" folder"+
			"\nor the option \"-Djava.library.path\" is correctly set,"+
			"\notherwise all output messages will not be send."+
			"\n*** WARNING ***\n";
			
			System.out.println(s);
		}
	}

	/* declare the JNI method
	 */
	public static native boolean isPCICard();
	public native void sendToPCICard( double output, double value );
	public native void sendToSTDOutput
		( String time, String name, int output, int value );
	
	////////////////////////////////////////////
	
	/* create a project by default
	 */
	public Project() {
		for( int i=0; i<24; i++ ) {
			
			// set default colors
			Color color = DEFAULT_TRACK_COLOR;
			/*switch( i/6 ) {
				case 0 : color= COLOR_GREEN; break;
				case 1 : color= COLOR_YELLOW; break;
				case 2 : color= COLOR_PURPLE; break;
				case 3 : color= COLOR_BLUE; break;
			}*/
			
			// create track
			setTrack( i, true, true, color, DEFAULT_TRACK_NAME + ("."+(i+1)) );
		}
	}
	
	/* launch a existent project
	 * @arg File, the project file
	 */
	public Project( File f ) {
		
		setFile( f );
		
		// parse XML content
		System.out.print( "load file \""+f.getName()+"\" ... " );
		try {
			
			XML xml = new XML();
			xml.open( f );
			
			// set project properties
			setName( xml.getRootAttribut("name") );
			setDuration( Double.parseDouble(xml.getRootAttribut("duration")) );
			setVideo( xml.getRootAttribut("video") );
			
			// create tracks objects
			int num = xml.getLength();
			for( int i=1; i<num; i+=2 ) {
				
				Track track		= new Track();
				String[] color	= xml.getTagAsObject(i,"color");
				String[] output	= xml.getTagAsObject(i, "outputs");
				
				// set properties
				track.setName( xml.getTagValue(i,"name") );
				track.setIndex( Integer.parseInt(xml.getIDAttribut(i,"id")) );
				track.setEnabled( xml.getTagValue(i,"enable").equals("true") );
				track.setVisible( xml.getTagValue(i,"visible").equals("true") );
				track.setColor( new Color( Integer.parseInt(color[0]),
									 Integer.parseInt(color[1]),
									 Integer.parseInt(color[2])
									)
							);
				track.setLength( duration );
				
				// set outputs
				for( int j=0; j<output.length; j+=2)
					track.add( 
						Double.parseDouble(output[j]),
						Double.parseDouble(output[j+1])
					);
				track.deselect();
				//System.out.println( "init selection:"+track.getSelectionIndex() );
				
				// add track reference
				tracks.add( track );
			}
			
			// finalyze
			System.out.println( "ok" );
		}
		catch( Exception e ) { 
			System.out.println( "failed!\n\t> "+e.getMessage() );
		}
	}
	
	/* get/set the project File
	 * @arg File or String, the project file
	 */
	public void setFile( File f )	{ file = f; }
	public void setFile( String s ) { file = new File(s); }
	public File getFile() { return file; }
	
	/* get/set the project video File
	 * @arg String, the video file path
	 */
	public void setVideo( String s ) {
		if ( !s.equals("")) video = new File(s);
		else video = null;
	}
	public File getVideo() { return video; }
	
	/* get/set the project name
	 * @arg String, the new project name
	 */
	public void setName( String s ) { name = s.replaceAll( ".xml", "" ); }
	public String getName() { return name; }
	
	/* get/set the project duration
	 */
	public void setDuration( double d ) { duration = d; }
	public double getDuration() { return duration; }
	
	/* set, update, or create track (properties)
	 * @arg int, the track number
	 * @arg boolean, the track enable status
	 * @arg boolean, the track visible status
	 * @arg Color, the track color
	 * @arg String, the track name
	 */
	public void setTrack
	( int idx, boolean isEnabled, boolean isVisible, Color color, String name )
	{
		Track track = new Track();
		
		track.setIndex( idx );
		track.setEnabled( isEnabled );
		track.setVisible( isVisible );
		track.setColor( color );
		track.setName( name) ;
		track.setLength( duration );
		// create a new track if needed,
		// or update an existing track
		if ( idx==getTrackCount() ) tracks.add( track );
		else {
			tracks.set( idx, track );
			reassign( idx ); // update indexes
		}
	}
	
	/* return the number of tracks
	 */
	public int getTrackCount() {
		return tracks.size();
	}
	
	/* return the required track object
	 * @arg int, the track index
	 */
	public Track getTrack( int idx ) {
		return (Track)tracks.get(idx);
	}
	
	/* remove the specified track
	 * @arg, the track index to be removed
	 */
	public void removeTrack( int idx ) {
		tracks.remove( idx );
		reassign( idx );
	}
	
	/* reassign tracks indexes
	 * @arg int, check from this value
	 */
	private void reassign( int from ) {
		for(int i=from; i<tracks.size(); i++) {
			Track t = getTrack(i);
			if ( t.getIndex()!=i ) t.setIndex(i);
		}
	}
	
	/* return the current chrono value in second
	 */
	public synchronized double getChrono() {
		return chrono;
	}
	
	/* update chrono value and free the threads 
	 * which await the value of chrono
	 * @arg double, the new chrono value in second
	 */
	public synchronized void setChrono( double d ) {
		
		// SYNCHRONIZED WITH THE VIDEO ?
		
		chrono = d;
		notifyAll();
	}
	
	/* return the current timer status
	 */
	public boolean isRunning() {
		return ( timer!=null );
	}
	
	/* start running
	 */
	public void start() {
		System.out.println( "\nProject \"" + name + 
							"\" session started at " + new Date());
		
		// prepare resources
		//prepare( chrono );
		
		// init the timer
		if ( isRunning() ) stop();
		timer = new Timer();
		timer.scheduleAtFixedRate( new Process(), 0, TIMER_PERIOD );
	}
	
	/* stop running
	 */
	public void stop() {
		
		reset();
		
		timer.cancel();
		timer = null;
		System.out.println( "session stop at " + new Date() );
	}
	
	/* timer task :
	 * run this project, play the video, send output message
	 */
	class Process extends TimerTask {
		
		// store each track status true=on/false=off
		boolean status[];
		
		/* init status
		 */
		public Process() {
			status = new boolean[ getTrackCount() ];
			
			// if chrono=.0 all output are normally off
			/*if ( chrono!=.0 )
				for( int i=0; i<status.length; i++ ) 
					status[i] = getTrack(i).checkStatus(chrono);*/
		}
		
		/* run method
		 */
		public synchronized void run() {
			
			String time = Toolbar.TimeFormat.format( chrono );
			
			// send outputs
			for( int i=0; i<status.length; i++ ) {
				Track track = getTrack(i);
				if ( track.enable ) {//&& !track.done ) {
					
					// FIND ANOTHER WAY FOR NOT CHECK EACH SECOND
					// check for the current status
					boolean on = track.checkStatus(chrono);
					if ( on==status[i] ) continue;
					
					// update status and define the appropriete value
					status[i] = on;
					int value = on ? 1:0;
				
					// via JNI
					if ( isLIB ) {
						sendToSTDOutput( time, track.name, i, value);
						if ( toPCI ) sendToPCICard( (double)i,(double)value);
					}
					
					// via UDP
					// i = track number
					// 1=stop or 2=start
					try {
						String out	= i+","+value;
						byte[] buf	= out.getBytes();
						DatagramPacket pa = new DatagramPacket(
												buf, 
												buf.length,
												InetAddress.getByName(""),
												PORT
											);
						if ( socket!=null ) socket.send( pa );
					}
					catch( IOException e) { 
						System.out.println( e.getMessage() );
					}
				}
			}
			
			// update chrono
			setChrono( chrono + 1.0 );
			if ( chrono > duration ) stop();
		}
	}
	
	
	
	/* save project as XML document
	 */
	public void save() {
		
		XML xml  = new XML();
		String f = video==null ?  "" : video.getAbsolutePath();
		System.out.print( "saving \""+ name + "\" ... " );
		
		// write XML content
		xml.create( file );
		xml.writeln( "<!-- DO NOT EDIT THIS FILE - it is machine generated -->" );
		xml.write( "<project name=\""+getName()+"\" ");
		xml.write( "duration=\""+duration+"\" " );
		xml.write( "video=\""+f+"\"" );
		xml.writeln( ">" );
		
		// tracks value
		for( int i=0; i<tracks.size(); i++ ) {
			
			Track t = getTrack(i);
			Color c	= t.color;
			double[] outputs = t.getOutputs();
			
			xml.writeln( "\t<track id=\""+ t.index + "\">" );
			xml.writeln( "\t\t<name>"+ t.name + "</name>");
			xml.writeln( "\t\t<enable>"+ t.enable + "</enable>");
			xml.writeln( "\t\t<visible>"+ t.visible + "</visible>");
			xml.writeln( "\t\t<color>" + 
					   c.getRed() + "," + 
					   c.getGreen() + "," + 
					   c.getBlue() + "</color>" 
						 );
			xml.write( "\t\t<outputs>" );
			for( int j=0; j<outputs.length; j +=2 ) {
				xml.write( outputs[j]+","+outputs[j+1] );
				if (j<outputs.length-2) xml.write( "," );
			}
			xml.writeln( "</outputs>" );
			xml.writeln( "\t</track>" );
		}
		
		xml.writeln( "</project>" );
		xml.close();
		
		
		System.out.println( "ok" );
	}
	
	/* Stop all ouput via UDP
	 */
	public static void reset() {
		try {
			System.out.print( "reset outputs ... " );
			//if( socket==null ) socket = new DatagramSocket( PORT+100 );
			for( int i=0; i<24; i++ ) {
				byte[] buf	= (i+",0").getBytes();
				DatagramPacket pa = new DatagramPacket(
										   buf, 
										   buf.length,
										   InetAddress.getByName(""),
										   PORT
									);
				socket.send( pa );
			}
			//socket.close();
			//socket = null;
			System.out.println( "ok" );
		}
		catch( IOException e) { System.out.print( "failed! "+e.getMessage() ); }
	}
}
