/*
 * (./) Video.java 12/4/05 by cs for "MMM" project
 * (cc) some right reserved
 */

package mmm.media;

import java.awt.*;
import javax.swing.JComponent;
import javax.media.*;
import javax.media.bean.playerbean.*;//.MediaPlayer;

import com.sun.media.JMFSecurityManager;

/* the video component 
 */
public class Video extends JComponent implements ControllerListener {
	
    private MediaPlayer player	= null;
	private boolean isPlaying	= false;
	private boolean	enable		= false;
	
	/**********************************
	SET AND IMPLEMENT A VARIABLE REPRESENTING IF THE PLAYER IS REALIZED/OK
	AND SYNCHRONOZIED ALL ACTION TO THIS VALUE > done or error?
	***********/
	
	/**
	 * Check for JMF native libraries
	 */
	static {
		// CHeck for JMF Native Libraries on Win32
		if ( System.getProperty("os.name").indexOf("Windows")!=-1 ) {
			try { JMFSecurityManager.loadLibrary( "jmutil" ); }
			catch( UnsatisfiedLinkError e ) {
				System.out.println("\n*** WARNING ***\n"+
								   "JMF native libraries could not be found, "+
								   "the video will be not played correctly.\n"+
								   "Verify that the Djava.library.path option "+
								   "include the JMF shared libraries"+
								   "\n*** WARNING ***\n"
								   );
			}
		}
	}
	
    /* init component
	 */
    public Video() {
		
		// create player
		player = new MediaPlayer();
		player.setBackground( Color.BLACK );
		player.setPlaybackLoop ( false );
		player.setFixedAspectRatio ( false );
		player.setControlPanelVisible( false );
		player.addControllerListener( this );
		player.setPopupActive( false );
		
		// component
		setLayout( new BorderLayout() );
		setBounds( 0, 0, 320, 240 );
		setPreferredSize( new Dimension( 320, 240 ) );
    }
    
	/* paint the component background
	 */
	public void paint( Graphics g ) {
		g.setColor( Color.BLACK );
		g.fillRect(0, 0, getWidth(), getHeight() );
	}
	
	/* load a new video in the player
	 * @arg String, the video file path
	 * return true if no error
	 */
	public synchronized boolean load( String f ) {
		
		// clear all resources if needed 
		if ( enable ) clear();
		
		// prefecth movie
		enable = false;
		player.setVisible( true );
		player.setMediaLocation( f );
		try {
			player.realize();
			System.out.print( "fetching movie \""+f+"\" ... ");
			wait();
		}
		catch( InterruptedException e ) {;}
		
		return enable;
	}
	
	/* prefetch the realized video
	 */
	public void prefetch() {
		if ( enable ) player.prefetch();
	}
	
	/* remove the player resources
	 */
	public void clear() {
		
		String f = player.getMediaLocation();
		
		if ( !f.equals("") ) {
			System.out.println( "close movie \""+f+"\"" );
			player.deallocate();
			player.close();
			player.setVisible( false );
		}
	}
	
	/* start playing
	 * @arg long, the start time
	 */
	public void start() { start(0.); }
	public void start( double time ) {
		if ( enable ) {
			player.syncStart( new Time(time) );
			isPlaying = true;
			player.setVisible( true );
		}
	}
	
	/* stop playing
	 */
	public void stop() {
		if ( enable ) {
			//player.stop();
			player.stopAndDeallocate();
			isPlaying = false;
		}
	}
	
	/* set the video media time
	 * @arg double, the required time value as milliseconds
	 */
	public void setTime( double d ) {
		if ( enable ) {
			
			// hide/show player
			boolean end = (d > player.getDuration().getSeconds());
			player.setVisible( !end );
			if ( end ) return;
			
			player.setMediaTime( new Time(d) );
			if ( isPlaying ) {
				stop();
				start();
			}
		}
	}
		
	/* return the current player component
	 * @type Component
	 * @throw NullPointerException
	 */
	public MediaPlayer getVideoComponent() throws NullPointerException {
		if ( player.getVisualComponent()==null ) 
			throw new NullPointerException( "no visual player found" );
		return player;
	}
	
	/* return the current video duration
	 * @type Time
	 */
	public Time getDuration() {
		return player.getDuration();
	}
	
	/* controller listener
	 */
    public synchronized void controllerUpdate( ControllerEvent e ) {
		
		// ConfigureCompleteEvent
		/*if ( e instanceof ConfigureCompleteEvent) {
			System.out.println( "ConfigureCompleteEvent: "+e );
		}*/
		
		// unavailable formats
		if ( e instanceof ResourceUnavailableEvent) {
			// finalize load() method
			System.out.println( "failed!\n\t>"+
								((ResourceUnavailableEvent)e).getMessage()
								);
			remove( player );
			notify();
		}
		
		// RealizeCompleteEvent
		if ( e instanceof RealizeCompleteEvent ) {
			// finalize load() method
			System.out.println( "ok" );
			enable = true;
			notify();
			prefetch();
		}
		
		// PrefetchCompleteEvent
		if ( e instanceof PrefetchCompleteEvent) {
			add( player );
			player.repaint();
			//validate();
		}
		
		// EndOfMediaEvent
		if ( e instanceof EndOfMediaEvent ) {
			player.setVisible( false );
		}
    }
}
