/*
 * (./) FullScreenWindow.java 12/12/05 by cs for "MMM" project
 * (cc) some right reserved
 */

package mmm.ui;

import java.awt.*;
import javax.swing.*;
import java.awt.image.MemoryImageSource;

//import mmm.media.Video;


/* a JWindow extends class to display the video in full screen mode
 */
public class FullScreenWindow extends JWindow {
	
	// ...
	private Toolkit toolkit			= Toolkit.getDefaultToolkit();
	private GraphicsDevice device	= GraphicsEnvironment.getLocalGraphicsEnvironment().
										getDefaultScreenDevice();
	
	// cursor
    private Cursor csrDefault = new Cursor( Cursor.DEFAULT_CURSOR );
    private Cursor csrHidden  = toolkit.createCustomCursor(
									toolkit.createImage(
										new MemoryImageSource(1,1,new int[1],0,1)
									),
									new Point(0, 0),
									"HIDDEN_CURSOR"
								);
	
	// the parent frame
	private JFrame owner = null;
	

	/* constructor
	 */
	public FullScreenWindow() { init(); }
	public FullScreenWindow( JFrame owner ) { super(owner); init(); }
	
	/* init window
	 */
	public void init() {
		setBackground( Color.BLACK );
		getContentPane().setLayout( new BorderLayout() );
	}
	
	/* display the in full-screen mode
	 * @arg Video, the video component owner to get and remove the video player
	 */
	public void enter( Component comp ) {
		
		// check for error
		if ( !device.isFullScreenSupported() ) {
			System.out.println( "Fullscreen is not supported by this device" );
			return;
		}
		
		toolkit.sync();
		device.setFullScreenWindow( this );
		
		try {
			/*Component comp = video.getVideoComponent();
			video.remove( comp );
			video.validate();*/
			this.getContentPane().add( comp );
		}
		catch( NullPointerException e ) {;}
		finally{
			if ( owner!=null) owner.setVisible( false );
			this.setVisible( true );		
			this.setCursor( csrHidden );
			this.validate();
		}
    }
	
	/* exit the full-screen mode
	 * @arg Video, the video component owner to reasign the video player
	 */
	public void exit( Component comp ) {
		if ( device.isFullScreenSupported() ) {
			
			toolkit.sync();
			device.setFullScreenWindow( null );
			
			try {
				/*Component comp = video.getVideoComponent();
				video.add( comp );
				video.validate();*/
				this.remove( comp );
			}
			catch( NullPointerException e ) {;}
			finally{
				if ( owner!=null) owner.setVisible( true );
				this.setCursor( csrDefault );
				this.setVisible( false );
				this.validate();	
			}
		}
    }
}
